@@ -6,6 +6,7 @@ mod codegen;
 #[cfg(feature = "graphviz")]
 mod diagramgen;
 mod parser;
+mod validation;
 
 use syn::parse_macro_input;
 
@@ -51,6 +52,11 @@ pub fn statemachine(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
                 }
             }
 
+            // Validate the parsed state machine before generating code.
+            if let Err(e) = validation::validate(&sm) {
+                return e.to_compile_error().into();
+            }
+
             codegen::generate_code(&sm).into()
         }
         Err(error) => error.to_compile_error().into(),
